package tests;

import entities.nomapkeyannotationwithmanytomany.TheatreCompany;
import entities.nomapkeyannotationwithmanytomany.TheatreLocation;
import java.util.HashMap;
import java.util.Map;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

public class TestNoMapKeyAnnotationManyToMany {
    
    private String testID;
    private EntityManagerFactory emf;
    
    private static long REGAL_ID = 676l;
    private static String REGAL_NAME = "Regal";
    private static long KNOXVILLE_ID = 42l;
    private static String KNOXVILLE_CODE = "KXTN";
    private static long IOWA_CITY_ID = 47l;
    private static String IOWA_CITY_CODE = "ICIO";
    
    public TestNoMapKeyAnnotationManyToMany() {
    }
    
    public void test() {
        emf = Persistence.createEntityManagerFactory("mapkey-bug");
        persistRelationship();
        assertRelationshipPersisted();
    }
    
    private void persistRelationship() {
        try {
            TheatreCompany regal = createTheatreCompany(REGAL_ID, REGAL_NAME);
            TheatreLocation knoxville = createTheatreLocation(KNOXVILLE_ID, KNOXVILLE_CODE);
            TheatreLocation iowacity = createTheatreLocation(IOWA_CITY_ID, IOWA_CITY_CODE);
            
            Map<String, TheatreLocation> regalLocations = new HashMap<String, TheatreLocation>();
            regalLocations.put(KNOXVILLE_CODE, knoxville);
            regalLocations.put(IOWA_CITY_CODE, iowacity);
            
            regal.setLocations(regalLocations);
            knoxville.addCompany(regal);
            iowacity.addCompany(regal);
            
            EntityManager em = emf.createEntityManager();
            em.getTransaction().begin();
            em.persist(regal);
            em.getTransaction().commit();
            em.close();
            
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    private void assertRelationshipPersisted() {
        try {
            EntityManager em = emf.createEntityManager();
            
            TheatreCompany retrievedRegal = em.find(TheatreCompany.class, REGAL_ID);
            Map<String, TheatreLocation> regalLocations = retrievedRegal.getLocations();
            
            
            // CODE as the key !!
            
            if ((regalLocations.get(KNOXVILLE_CODE).getId() == KNOXVILLE_ID) &&
                    (regalLocations.get(IOWA_CITY_CODE).getId() == IOWA_CITY_ID) ) {
                System.out.println("TestNoMapKeyManyToMany Passed - Able to read values from the associated Map using \'Code\' as the key");
            } else
                System.out.println("TestNoMapKeyManyToMany Failed - Able to read values from the associated Map using \'Code\' as the key; But values read do not match");
            
            // ID as the key !!
            
//            if ((regalLocations.get(KNOXVILLE_ID).getCode().equals(KNOXVILLE_CODE)) &&
//                    (regalLocations.get(IOWA_CITY_ID).getCode().equals(IOWA_CITY_CODE)) ) {
//                System.out.println("Test Passed - Able to read values from the associated Map using \'id\' as the key");
//            } else
//                System.out.println("Test Failed - Able to read values from the associated Map using \'id\' as the key; But values read do not match");
            
        } catch (NullPointerException e) {
            System.out.println("TestNoMapKeyManyToMany Failed - Unable to read values from the associated collection using \'Code\' as the Key");
            e.printStackTrace();
        }
    }
    
    private TheatreLocation createTheatreLocation(long id, String code) {
        TheatreLocation loc = new TheatreLocation();
        loc.setId(id);
        loc.setCode(code);
        return loc;
    }
    
    private TheatreCompany createTheatreCompany(long id, String name) {
        TheatreCompany company = new TheatreCompany();
        company.setId(id);
        company.setName(name);
        return company;
    }
    
}
