package tests;

import entities.overridemapkeyannotationonetomany.Course;
import entities.overridemapkeyannotationonetomany.Student;
import java.util.HashMap;
import java.util.Map;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;

public class TestOverrideMapKeyOneToMany {
    
    private String testID;
    private EntityManagerFactory emf;
    
    private static int STUDENT_ID1 = 420;
    private static String STD_NAME1 = "Charsoubees";
    private static int STUDENT_ID2 = 421;
    private static String STD_NAME2 = "CharsouIkkis";
    
    private static  int MATH_ID = 104;
    private static String MATH_COURSE = "Math";
    
    public TestOverrideMapKeyOneToMany() {
    }
    
    public void test() {
        try {
            emf = Persistence.createEntityManagerFactory("mapkey-bug");
            persistRelationship();
            assertRelationshipPersisted();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    private void persistRelationship() {
        EntityManager em = emf.createEntityManager();
        EntityTransaction tx = em.getTransaction();
        
        tx.begin();
        
        Course math_course = createCourse(MATH_ID, MATH_COURSE);
        Student studentInMath_1 = createStudent(STUDENT_ID1, STD_NAME1, math_course);
        Student studentInMath_2 = createStudent(STUDENT_ID2, STD_NAME2, math_course);
        
        Map mathStudents = new HashMap();
        mathStudents.put(STUDENT_ID1, studentInMath_1);
        mathStudents.put(STUDENT_ID2, studentInMath_2);
        
        math_course.setStudents(mathStudents);
        
        em.persist(math_course);
        em.persist(studentInMath_1);
        em.persist(studentInMath_2);
        
        tx.commit();
        em.close();
    }
    
    private void assertRelationshipPersisted() throws Exception {
        try {
            EntityManager em = emf.createEntityManager();
            
            Course retrievedMath = em.find(Course.class, MATH_ID);
            
            Map<String, Student> mathStudents = retrievedMath.getStudents();
            if ( (mathStudents.get(STD_NAME1).getId() == STUDENT_ID1) &&
                    (mathStudents.get(STD_NAME2).getId() == STUDENT_ID2) ) {
                System.out.println("TestOverrideMapKeyOneToMany Passed - Able to read values from the associated Map using \'Name\' as the key");
            } else
                System.out.println("TestOverrideMapKeyOneToMany Failed - Able to read values from the associated Map using \'Name\' as the key; But values read do not match");
        } catch(NullPointerException ex) {
            System.out.println("TestOverrideMapKeyOneToMany Failed - Unable to read values from the associated collection using \'Name\' as the Key");
            ex.printStackTrace();
        }
    }
    
    private Student createStudent(int id, String name, Course course) {
        Student student = new Student();
        student.setId(id);
        student.setName(name);
        student.setCourse(course);
        return student;
    }
    
    private Course createCourse(int id, String courseName) {
        Course course = new Course();
        course.setId(id);
        course.setName(courseName);
        return course;
    }
    
}
